#methods implemented:  con, bil, bic, nn, con2
#target_grid has to be either a grid recognised by CDO or a NETCDF file
# requires CDO_1.9.8 or newer versions when using nn method
source('/esarchive/scratch/jramon/GitLab_jramon/downscaling/csdownscale/R/create_interp_weights.R')
source('/esarchive/scratch/jramon/GitLab_jramon/downscaling/csdownscale/R/interpolate_model_data.R')
Interpolation <- function(exp, method, target_grid = NULL, points = NULL) {

  require(s2dv)

  # Input data must be an s2dv_cube object
  if (!inherits(exp,'s2dv_cube')) {
    stop("Parameter 'exp' must be of the class 's2dv_cube'")
  }

  if (!is.null(points)) {
    if (!inherits(points, 'list')) {
      stop("Parameter 'points' must be a list of two elements containing the point ",
           "latitudes and longitudes in the form 'list(c(latitudes), c(longitudes))'")
    }    

    # Check that the number of latitudes and longitudes match
    stopifnot(length(unique(lengths(points))) == 1L)
  }

  lonmin <- attr(exp$lon,"first_lon")
  lonmax <- attr(exp$lon,"last_lon")
  latmin <- attr(exp$lat,"first_lat")
  latmax <- attr(exp$lat,"last_lat")

  # Map regrid with CDO
  if (is.null(points)) {
    res <- CDORemap(data_array = exp$data, 
                    lats = exp$lat, 
                    lons = exp$lon, 
                    grid = target_grid, 
                    method = method, 
                    crop = c(lonmin, lonmax, latmin, latmax))

  # Interpolate to point locations
  } else {  
    # First create interpolation weights, depending on the chosen method
    weights <- create_interp_weights(ncfile = exp$source_files[1], locids = 1:unique(lengths(points)), 
                                     lats = points[[1]], lons = points[[2]], method = method) 

    # Select coarse-scale data to be interpolated
    model_data_gridpoints <- get_model_data(weights.df = weights, mdata = exp$data)

    # Interpolate model data to point locations
    res <- interpolate_data(model_data_gridpoints, weights)
  }
 
  # Create an s2dv_cube object
  res_s2dv <- suppressWarnings(s2dv_cube(data = res$data_array, lon = res$lons, lat = res$lats))

  return(res_s2dv)
}




get_model_data <- function(weights.df, mdata)

                #----------------------------------
                # Compute interpolation weights. Get grid from first file
                #----------------------------------
                rea.path <- grep(reanalyses[rea],reanalyses_paths,value=T)
                ncfirst <- paste(rea.path,paste0(variable,"_",s1,"01.nc"),sep="/")
                weights <- create_interp_weights(ncfile = ncfirst,locids = metadata$Name, lats = metadata$Lat,
                                                 lons = metadata$Lon, method = method)

                #----------------------------------
                # Load data at (i,j) locations
                # sdates must be in format yyyymm
                #----------------------------------
                sdates  <- format(ymd(paste0(s1,"0101")) + months(0:11) + rep(years(0:(s2-s1)),each=12),"%Y%m")
                nsdates <- length(sdates)

                files   <- paste(rea.path,"$var$_$sdate$.nc",sep="/")
                model_data <- get_model_data(weights,dat=files,var=variable,sdate=sdates,time='all',num_procs=4)


